@@ -92,38 +92,17 @@ char *getProgramInitString(void)
 
 char *getConfigProgramTitleString(void)
 {
-  TreeInfo *graphics_current =
-    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
-
-  return (leveldir_current->program_title ?
-	  leveldir_current->program_title :
-	  graphics_current->program_title ?
-	  graphics_current->program_title :
-	  setup.internal.program_title);
+  return getConfigProgramString(TEXT_ID_PROGRAM_TITLE);
 }
 
 char *getConfigProgramCopyrightString(void)
 {
-  TreeInfo *graphics_current =
-    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
-
-  return (leveldir_current->program_copyright ?
-	  leveldir_current->program_copyright :
-	  graphics_current->program_copyright ?
-	  graphics_current->program_copyright :
-	  setup.internal.program_copyright);
+  return getConfigProgramString(TEXT_ID_PROGRAM_COPYRIGHT);
 }
 
 char *getConfigProgramCompanyString(void)
 {
-  TreeInfo *graphics_current =
-    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
-
-  return (leveldir_current->program_company ?
-	  leveldir_current->program_company :
-	  graphics_current->program_company ?
-	  graphics_current->program_company :
-	  setup.internal.program_company);
+  return getConfigProgramString(TEXT_ID_PROGRAM_COMPANY);
 }
 
 char *getWindowTitleString(void)
