@@ -20,6 +20,10 @@ void CreateLevelEditorGadgets(void);
 void FreeLevelEditorGadgets(void);
 void UnmapLevelEditorGadgets(void);
 void DrawLevelEd(void);
+
+boolean anyLevelEditorTextTypingActive(void);
+boolean anyLevelEditorTextTypingEndReached(void);
+boolean HandleLevelEditorTextTypingKeyInput(Key);
 void HandleLevelEditorKeyInput(Key);
 void HandleLevelEditorIdle(void);
 void HandleEditorGadgetInfoText(void *ptr);
@@ -28,13 +32,7 @@ void PrintEditorElementList(void);
 
 void DumpBrush(void);
 void DumpBrush_Small(void);
-
-void CopyClipboardToBrush(void);
-void CopyBrushToClipboard(void);
-void CopyBrushToClipboard_Small(void);
-
-void UndoLevelEditorOperation(void);
-void RedoLevelEditorOperation(void);
+void DumpBrush_Names(void);
 
 void SetDefaultLevelColorType_BD(void);
 void SetDefaultLevelColors_BD(void);
