@@ -24,15 +24,15 @@
 #include "config.h"
 
 
-#define	DEBUG_EVENTS		0
+#define	DEBUG_EVENTS			0
 
-#define DEBUG_EVENTS_BUTTON	(DEBUG_EVENTS	* 0)
-#define DEBUG_EVENTS_MOTION	(DEBUG_EVENTS	* 0)
-#define DEBUG_EVENTS_WHEEL	(DEBUG_EVENTS	* 1)
-#define DEBUG_EVENTS_WINDOW	(DEBUG_EVENTS	* 0)
-#define DEBUG_EVENTS_FINGER	(DEBUG_EVENTS	* 0)
-#define DEBUG_EVENTS_TEXT	(DEBUG_EVENTS	* 1)
-#define DEBUG_EVENTS_KEY	(DEBUG_EVENTS	* 1)
+#define DEBUG_EVENTS_BUTTON		(DEBUG_EVENTS	* 0)
+#define DEBUG_EVENTS_MOTION		(DEBUG_EVENTS	* 0)
+#define DEBUG_EVENTS_WHEEL		(DEBUG_EVENTS	* 1)
+#define DEBUG_EVENTS_WINDOW		(DEBUG_EVENTS	* 0)
+#define DEBUG_EVENTS_FINGER		(DEBUG_EVENTS	* 0)
+#define DEBUG_EVENTS_TEXT		(DEBUG_EVENTS	* 1)
+#define DEBUG_EVENTS_KEY		(DEBUG_EVENTS	* 1)
 
 
 static boolean cursor_inside_playfield = FALSE;
@@ -108,10 +108,8 @@ static int FilterEvents(const Event *event)
   {
     // do not reset mouse cursor before all pending events have been processed
     if (gfx.cursor_mode == cursor_mode_last &&
-	((game_status == GAME_MODE_TITLE &&
-	  gfx.cursor_mode == CURSOR_NONE) ||
-	 (game_status == GAME_MODE_PLAYING &&
-	  gfx.cursor_mode == CURSOR_PLAYFIELD)))
+	((game_status == GAME_MODE_TITLE   && gfx.cursor_mode == CURSOR_NONE) ||
+	 (game_status == GAME_MODE_PLAYING && gfx.cursor_mode == CURSOR_PLAYFIELD)))
     {
       SetMouseCursor(CURSOR_DEFAULT);
 
@@ -140,7 +138,8 @@ static int FilterEvents(const Event *event)
 
   // skip mouse motion events without pressed button outside level editor
   if (button_status == MB_RELEASED &&
-      game_status != GAME_MODE_EDITOR && game_status != GAME_MODE_PLAYING)
+      game_status != GAME_MODE_EDITOR &&
+      game_status != GAME_MODE_PLAYING)
     return 0;
 
   return 1;
@@ -335,8 +334,7 @@ static void HandleMouseCursor(void)
       SetMouseCursor(CURSOR_NONE);
     }
   }
-  else if (game_status == GAME_MODE_PLAYING && (!tape.pausing ||
-						tape.single_step))
+  else if (game_status == GAME_MODE_PLAYING && (!tape.pausing || tape.single_step))
   {
     // when playing, display a special mouse pointer inside the playfield
 
@@ -610,22 +608,22 @@ void HandleWindowEvent(WindowEvent *event)
   int subtype = event->event;
 
   char *event_name =
-    (subtype == SDL_WINDOWEVENT_SHOWN ? "SDL_WINDOWEVENT_SHOWN" :
-     subtype == SDL_WINDOWEVENT_HIDDEN ? "SDL_WINDOWEVENT_HIDDEN" :
-     subtype == SDL_WINDOWEVENT_EXPOSED ? "SDL_WINDOWEVENT_EXPOSED" :
-     subtype == SDL_WINDOWEVENT_MOVED ? "SDL_WINDOWEVENT_MOVED" :
-     subtype == SDL_WINDOWEVENT_SIZE_CHANGED ? "SDL_WINDOWEVENT_SIZE_CHANGED" :
-     subtype == SDL_WINDOWEVENT_RESIZED ? "SDL_WINDOWEVENT_RESIZED" :
-     subtype == SDL_WINDOWEVENT_MINIMIZED ? "SDL_WINDOWEVENT_MINIMIZED" :
-     subtype == SDL_WINDOWEVENT_MAXIMIZED ? "SDL_WINDOWEVENT_MAXIMIZED" :
-     subtype == SDL_WINDOWEVENT_RESTORED ? "SDL_WINDOWEVENT_RESTORED" :
-     subtype == SDL_WINDOWEVENT_ENTER ? "SDL_WINDOWEVENT_ENTER" :
-     subtype == SDL_WINDOWEVENT_LEAVE ? "SDL_WINDOWEVENT_LEAVE" :
-     subtype == SDL_WINDOWEVENT_FOCUS_GAINED ? "SDL_WINDOWEVENT_FOCUS_GAINED" :
-     subtype == SDL_WINDOWEVENT_FOCUS_LOST ? "SDL_WINDOWEVENT_FOCUS_LOST" :
-     subtype == SDL_WINDOWEVENT_CLOSE ? "SDL_WINDOWEVENT_CLOSE" :
-     subtype == SDL_WINDOWEVENT_TAKE_FOCUS ? "SDL_WINDOWEVENT_TAKE_FOCUS" :
-     subtype == SDL_WINDOWEVENT_HIT_TEST ? "SDL_WINDOWEVENT_HIT_TEST" :
+    (subtype == SDL_WINDOWEVENT_SHOWN		? "SDL_WINDOWEVENT_SHOWN" :
+     subtype == SDL_WINDOWEVENT_HIDDEN		? "SDL_WINDOWEVENT_HIDDEN" :
+     subtype == SDL_WINDOWEVENT_EXPOSED		? "SDL_WINDOWEVENT_EXPOSED" :
+     subtype == SDL_WINDOWEVENT_MOVED		? "SDL_WINDOWEVENT_MOVED" :
+     subtype == SDL_WINDOWEVENT_SIZE_CHANGED	? "SDL_WINDOWEVENT_SIZE_CHANGED" :
+     subtype == SDL_WINDOWEVENT_RESIZED		? "SDL_WINDOWEVENT_RESIZED" :
+     subtype == SDL_WINDOWEVENT_MINIMIZED	? "SDL_WINDOWEVENT_MINIMIZED" :
+     subtype == SDL_WINDOWEVENT_MAXIMIZED	? "SDL_WINDOWEVENT_MAXIMIZED" :
+     subtype == SDL_WINDOWEVENT_RESTORED	? "SDL_WINDOWEVENT_RESTORED" :
+     subtype == SDL_WINDOWEVENT_ENTER		? "SDL_WINDOWEVENT_ENTER" :
+     subtype == SDL_WINDOWEVENT_LEAVE		? "SDL_WINDOWEVENT_LEAVE" :
+     subtype == SDL_WINDOWEVENT_FOCUS_GAINED	? "SDL_WINDOWEVENT_FOCUS_GAINED" :
+     subtype == SDL_WINDOWEVENT_FOCUS_LOST	? "SDL_WINDOWEVENT_FOCUS_LOST" :
+     subtype == SDL_WINDOWEVENT_CLOSE		? "SDL_WINDOWEVENT_CLOSE" :
+     subtype == SDL_WINDOWEVENT_TAKE_FOCUS	? "SDL_WINDOWEVENT_TAKE_FOCUS" :
+     subtype == SDL_WINDOWEVENT_HIT_TEST	? "SDL_WINDOWEVENT_HIT_TEST" :
      "(UNKNOWN)");
 
   Debug("event:window", "name: '%s', data1: %ld, data2: %ld",
@@ -718,8 +716,7 @@ static struct
   byte action;
 } touch_info[NUM_TOUCH_FINGERS];
 
-static void SetTouchInfo(int pos, SDL_FingerID finger_id, int counter,
-			 Key key, byte action)
+static void SetTouchInfo(int pos, SDL_FingerID finger_id, int counter, Key key, byte action)
 {
   touch_info[pos].touched = (action != JOY_NO_ACTION);
   touch_info[pos].finger_id = finger_id;
@@ -743,10 +740,8 @@ static void HandleFingerEvent_VirtualButtons(FingerEvent *event)
   int grid_button = overlay.grid_button[x][y];
   int grid_button_action = GET_ACTION_FROM_GRID_BUTTON(grid_button);
   Key key = GetKeyFromGridButton(grid_button);
-  int key_status = (event->type == EVENT_FINGERRELEASE ? KEY_RELEASED :
-		    KEY_PRESSED);
-  char *key_status_name = (key_status == KEY_RELEASED ? "KEY_RELEASED" :
-			   "KEY_PRESSED");
+  int key_status = (event->type == EVENT_FINGERRELEASE ? KEY_RELEASED : KEY_PRESSED);
+  char *key_status_name = (key_status == KEY_RELEASED ? "KEY_RELEASED" : "KEY_PRESSED");
   int i;
 
   // for any touch input event, enable overlay buttons (if activated)
@@ -1344,8 +1339,7 @@ static void HandleButtonOrFinger_FollowFinger(int mx, int my, int button)
     }
     else if (dx != 0 || dy != 0)
     {
-      if (player_is_dropping &&
-	  player_drop_count == getPlayerInventorySize(0))
+      if (player_is_dropping && player_drop_count == getPlayerInventorySize(0))
       {
 	Debug("event:finger", "---------- DROP -> SNAP ----------");
 
@@ -1535,7 +1529,7 @@ static int HandleDropFileEvent(char *filename)
     return TREE_TYPE_UNDEFINED;
   }
 
-  if (tree_type == TREE_TYPE_LEVEL_DIR &&
+  if (tree_type == TREE_TYPE_LEVELSET_DIR &&
       game_status == GAME_MODE_LEVELS &&
       leveldir_current->node_parent != NULL)
   {
@@ -1548,7 +1542,7 @@ static int HandleDropFileEvent(char *filename)
     // use private level directory instead of top-level package level directory
     if (strPrefix(directory, options.level_directory) &&
 	strEqual(leveldir_current->node_parent->fullpath, "."))
-      directory = getUserLevelDir(NULL);
+      directory = getUserLevelsetDir(NULL);
   }
 
   // check if level or artwork set directory already exists in target directory
@@ -1558,7 +1552,7 @@ static int HandleDropFileEvent(char *filename)
     char message[100];
 
     sprintf(message, "%s set already exists! Replace with new set?",
-            (tree_type == TREE_TYPE_LEVEL_DIR ? "Level" : "Artwork"));
+            (tree_type == TREE_TYPE_LEVELSET_DIR ? "Level" : "Artwork"));
 
     // ask if level or artwork set directory should be replaced (and fail if not)
     if (!Request(message, REQ_ASK))
@@ -1613,8 +1607,7 @@ static int HandleDropFileEvent(char *filename)
   // force restart after adding level collection
   if (getTreeInfoFromIdentifier(TREE_FIRST_NODE(tree_type), top_dir) == NULL)
   {
-    Request("Program must be restarted after adding a new level collection!",
-	    REQ_CONFIRM);
+    Request("Program must be restarted after adding a new level collection!", REQ_CONFIRM);
 
     CloseAllAndExit(0);
   }
@@ -1653,8 +1646,7 @@ static void HandleDropCompleteEvent(int num_level_sets_succeeded,
     sprintf(message_part1, "New %s set%s added",
 	    (num_artwork_sets_succeeded == 0 ? "level" :
 	     num_level_sets_succeeded == 0 ? "artwork" : "level and artwork"),
-	    (num_level_sets_succeeded +
-	     num_artwork_sets_succeeded > 1 ? "s" : ""));
+	    (num_level_sets_succeeded + num_artwork_sets_succeeded > 1 ? "s" : ""));
 
     if (num_files_failed > 0)
       sprintf(message, "%s, but %d dropped file%s failed!",
@@ -1664,8 +1656,7 @@ static void HandleDropCompleteEvent(int num_level_sets_succeeded,
   }
   else if (num_files_failed > 0)
   {
-    sprintf(message, "Failed to process dropped file%s!",
-	    num_files_failed > 1 ? "s" : "");
+    sprintf(message, "Failed to process dropped file%s!", num_files_failed > 1 ? "s" : "");
   }
 
   Request(message, REQ_CONFIRM);
@@ -1700,7 +1691,7 @@ void HandleDropEvent(Event *event)
     {
       int tree_type = HandleDropFileEvent(event->drop.file);
 
-      if (tree_type == TREE_TYPE_LEVEL_DIR)
+      if (tree_type == TREE_TYPE_LEVELSET_DIR)
 	num_level_sets_succeeded++;
       else if (tree_type == TREE_TYPE_GRAPHICS_DIR ||
 	       tree_type == TREE_TYPE_SOUNDS_DIR ||
@@ -1890,7 +1881,7 @@ void HandleButton(int mx, int my, int button, int button_nr)
 
 #ifdef DEBUG
       if (button == MB_PRESSED && !motion_status && !button_hold &&
-	  IN_GFX_FIELD_PLAY(mx, my) && GetKeyModState() & KMOD_Control)
+	  IN_GFX_FIELD_PLAY(mx, my) && (GetKeyModState() & KMOD_Control))
 	DumpTileFromScreen(mx, my);
 #endif
 
@@ -2015,33 +2006,23 @@ static void HandleKeysSpecial(Key key)
     {
       DumpBrush();
     }
-    else if (strSuffix(cheat_input, ":DDB"))
+    else if (strSuffix(cheat_input, ":dump-brush-small") ||
+	     strSuffix(cheat_input, ":DDB"))
     {
       DumpBrush_Small();
     }
-
-    if (GetKeyModState() & (KMOD_Control | KMOD_Meta))
+    else if (strSuffix(cheat_input, ":dump-brush-names") ||
+	     strSuffix(cheat_input, ":NDB"))
     {
-      if (letter == 'x')	// copy brush to clipboard (small size)
-      {
-	CopyBrushToClipboard_Small();
-      }
-      else if (letter == 'c')	// copy brush to clipboard (normal size)
-      {
-	CopyBrushToClipboard();
-      }
-      else if (letter == 'v')	// paste brush from Clipboard
-      {
-	CopyClipboardToBrush();
-      }
-      else if (letter == 'z')	// undo or redo last operation
-      {
-	if (GetKeyModState() & KMOD_Shift)
-	  RedoLevelEditorOperation();
-	else
-	  UndoLevelEditorOperation();
-      }
+      DumpBrush_Names();
     }
+#ifdef DEBUG
+    else if (strSuffix(cheat_input, ":test-add-to-clipboard") ||
+	     strSuffix(cheat_input, ":tatc"))
+    {
+      SDL_SetClipboardText("Test\r\nTest\r\n\r\nTest\r\n");
+    }
+#endif
   }
 
   // special key shortcuts for all game modes
@@ -2062,13 +2043,22 @@ static boolean HandleKeysSpeed(Key key, int key_status)
 	key == setup.shortcut.speed_slow)
     {
       int speed_factor = 4;
+      int speed_factor_fast = 1;
+      int speed_factor_slow = 1;
 
-      GameFrameDelay = (key_status != KEY_PRESSED ? setup.game_frame_delay :
-			key == setup.shortcut.speed_fast ? setup.game_frame_delay / speed_factor :
-			key == setup.shortcut.speed_slow ? setup.game_frame_delay * speed_factor :
-			setup.game_frame_delay);
+      if (key_status == KEY_PRESSED)
+      {
+        if (key == setup.shortcut.speed_fast)
+          speed_factor_fast = speed_factor;
+        else if (key == setup.shortcut.speed_slow)
+          speed_factor_slow = speed_factor;
+      }
+
+      GameFrameDelay = setup.game_frame_delay * speed_factor_slow / speed_factor_fast;
+      FfwdFrameDelay = FFWD_FRAME_DELAY       * speed_factor_slow / speed_factor_fast;
 
       GameFrameDelay = MIN(MAX(1, GameFrameDelay), 1000);
+      FfwdFrameDelay = MIN(MAX(1, FfwdFrameDelay), 1000);
 
       SetVideoFrameDelay(GameFrameDelay);
 
@@ -2142,6 +2132,97 @@ boolean HandleKeysDebug(Key key, int key_status)
   return FALSE;
 }
 
+static boolean HandleKeysClipboard(Key key)
+{
+  boolean text_input_gadgets_active = (anyTextInputGadgetActive() ||
+				       anyTextAreaGadgetActive());
+  boolean text_editor_typing_active = anyLevelEditorTextTypingActive();
+
+  // return if no text input gadget active and not typing text elements in level editor
+  if (!text_input_gadgets_active && !text_editor_typing_active)
+    return FALSE;
+
+  // return if neither Ctrl nor Meta/Cmd key pressed
+  if (!(GetKeyModState() & (KMOD_Control | KMOD_Meta)))
+    return FALSE;
+
+  if (key == KSYM_c && text_input_gadgets_active)
+  {
+    char *text = (anyTextInputGadgetActive() ? getTextFromActiveTextInputGadget() :
+		  anyTextAreaGadgetActive()  ? getTextFromActiveTextAreaGadget() :
+		  NULL);
+
+    if (text != NULL && strlen(text) > 0)
+    {
+      char *text_utf8 = getUTF8FromLatin1(text);
+
+      SDL_SetClipboardText(text_utf8);
+
+      checked_free(text_utf8);
+    }
+
+    return TRUE;
+  }
+  else if (key == KSYM_v)
+  {
+    if (!SDL_HasClipboardText())
+    {
+      Request("Clipboard is empty!", REQ_CONFIRM);
+
+      return FALSE;
+    }
+
+    char *text = SDL_GetClipboardText();
+    char *text_latin1 = getLatin1FromUTF8(text);
+    char *ptr = text_latin1;
+
+    while (*ptr)
+    {
+      char c = *ptr++;
+
+      if (c == '\r')	// ignore carriage return characters
+	continue;
+
+      char name[2] = { c, 0 };
+      Key key_from_clipboard = getKeyFromKeyName(name);
+
+      if (key_from_clipboard == KSYM_UNDEFINED)
+	key_from_clipboard = KSYM_question;
+
+      boolean success =
+	(text_input_gadgets_active ? HandleGadgetsKeyInput(key_from_clipboard) :
+	 text_editor_typing_active ? HandleLevelEditorTextTypingKeyInput(key_from_clipboard) :
+	 FALSE);
+
+      if (!success)
+	break;
+
+      if (text_editor_typing_active && anyLevelEditorTextTypingEndReached())
+	break;
+    }
+
+    checked_free(text_latin1);
+    SDL_free(text);
+
+    return TRUE;
+  }
+  else if (key == KSYM_x && text_input_gadgets_active)
+  {
+    // copy gadget text
+    HandleKeysClipboard(KSYM_c);
+
+    // clear gadget text
+    if (anyTextInputGadgetActive())
+      clearTextFromActiveTextInputGadget();
+    else if (anyTextAreaGadgetActive())
+      clearTextFromActiveTextAreaGadget();
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 void HandleKey(Key key, int key_status)
 {
   boolean anyTextGadgetActiveOrJustFinished = anyTextGadgetActive();
@@ -2497,6 +2578,9 @@ void HandleKey(Key key, int key_status)
 
   HandleKeysSpecial(key);
 
+  if (HandleKeysClipboard(key))
+    return;		// do not handle already processed keys again
+
   if (HandleGadgetsKeyInput(key))
     return;		// do not handle already processed keys again
 
@@ -2832,8 +2916,7 @@ void HandleJoystick(void)
   else
   {
     // first start with longer delay, then continue with shorter delay
-    joytest_delay.value =
-      (use_delay_value_first ? delay_value_first : delay_value);
+    joytest_delay.value = (use_delay_value_first ? delay_value_first : delay_value);
   }
 
   joytest_last = joytest;
@@ -2967,13 +3050,11 @@ void HandleSpecialGameControllerKeys(Key key, int key_status)
   {
     Event event;
 
-    event.type = (key_status == KEY_PRESSED ? SDL_CONTROLLERBUTTONDOWN :
-		  SDL_CONTROLLERBUTTONUP);
+    event.type = (key_status == KEY_PRESSED ? SDL_CONTROLLERBUTTONDOWN : SDL_CONTROLLERBUTTONUP);
 
     event.cbutton.which = 0;	// first joystick (Amazon Fire TV remote)
     event.cbutton.button = button;
-    event.cbutton.state = (key_status == KEY_PRESSED ? SDL_PRESSED :
-			   SDL_RELEASED);
+    event.cbutton.state = (key_status == KEY_PRESSED ? SDL_PRESSED : SDL_RELEASED);
 
     HandleJoystickEvent(&event);
   }
