@@ -35,6 +35,8 @@ void gd_game_free(GdGame *game)
     gd_cave_map_free(game->gfx_buffer);
   if (game->covered_buffer)
     gd_cave_map_free(game->covered_buffer);
+  if (game->scanned_next)
+    gd_cave_map_free(game->scanned_next);
 
   game->player_lives = 0;
 
@@ -65,19 +67,49 @@ static void add_bonus_life(GdGame *game, boolean inform_user)
   }
 }
 
+// this function is only needed to prevent variable name clash (should be fixed)
+static boolean game_has_no_time_limit(void)
+{
+  return game.no_level_time_limit;
+}
+
+// this function is only needed to prevent variable name clash (should be fixed)
+static boolean game_has_native_sound_engine(void)
+{
+  return game.use_native_bd_sound_engine;
+}
+
 // increment score of player.
 // flash screen if bonus life
 static void increment_score(GdGame *game, int increment)
 {
   int i;
 
-  i = game->player_score / gd_caveset_data->bonus_life_score;
+  if (game->state_counter == GAME_INT_CHECK_BONUS_TIME && game_has_no_time_limit())
+  {
+    // if not using time at all, do not change player score after playing
+    if (game->cave->no_time)
+      return;
+
+    // reverse time -- decrement player score by score for remaining time
+
+    game_bd.global_score = MAX(0, game_bd.global_score - increment);
+    game->player_score = MAX(0, game->player_score - increment);
+    game->cave_score = MAX(0, game->cave_score - increment);
+
+    return;
+  }
+
+  if (gd_caveset_data->bonus_life_score > 0)
+    i = game->player_score / gd_caveset_data->bonus_life_score;
+
   game_bd.global_score += increment;
   game->player_score += increment;
   game->cave_score += increment;
 
   // if score crossed bonus_life_score point boundary, player won a bonus life
-  if (game->player_score / gd_caveset_data->bonus_life_score > i)
+  if (gd_caveset_data->bonus_life_score > 0 &&
+      game->player_score / gd_caveset_data->bonus_life_score > i)
     add_bonus_life(game, TRUE);
 }
 
@@ -126,6 +158,11 @@ static void load_cave(GdGame *game)
     gd_cave_map_free(game->covered_buffer);
   game->covered_buffer = NULL;
 
+  // delete scanned next buffer
+  if (game->scanned_next)
+    gd_cave_map_free(game->scanned_next);
+  game->scanned_next = NULL;
+
   // load the cave
   game->cave_score = 0;
 
@@ -139,10 +176,6 @@ static void load_cave(GdGame *game)
 
   game->cave = gd_get_prepared_cave(game->original_cave, game->level_num);
 
-  // if requested, recolor cave (cave is a copy only, so no worries)
-  if (setup.bd_random_colors)
-    gd_cave_set_random_colors(game->cave, setup.bd_default_color_type);
-
   if (game->cave->intermission && game->cave->intermission_instantlife)
     add_bonus_life(game, FALSE);
 
@@ -204,6 +237,13 @@ static void load_cave(GdGame *game)
   for (y = 0; y < game->cave->h; y++)
     for (x = 0; x < game->cave->w; x++)
       game->covered_buffer[y][x] = FALSE;
+
+  // create scanned next buffer
+  game->scanned_next = gd_cave_map_new(game->cave, int);
+
+  for (y = 0; y < game->cave->h; y++)
+    for (x = 0; x < game->cave->w; x++)
+      game->scanned_next[y][x] = FALSE;
 }
 
 GdCave *gd_create_snapshot(GdGame *game)
@@ -233,10 +273,10 @@ GdGame *gd_game_new(const int cave, const int level)
   game->player_score = game_bd.global_score;
 
   game->player_move = GD_MV_STILL;
-  game->player_move_stick = FALSE;
+  game->player_move_sticky = game_bd.sticky_movement_input;
   game->player_fire = FALSE;
   game->player_suicide = FALSE;
-  game->player_suicide_stick = FALSE;
+  game->player_suicide_sticky = game_bd.sticky_movement_input;
 
   game->state_counter = GAME_INT_LOAD_CAVE;
 
@@ -290,12 +330,17 @@ static void iterate_cave(GdGame *game, GdDirection player_move, boolean fire, bo
       add_bonus_life(game, FALSE);
     }
 
+    // if not using time, set cave time to ten seconds for end jingle
+    if (game->cave->no_time)
+      game->cave->time = 10 * game->cave->timing_factor;
+
     // start adding points for remaining time
     game->state_counter = GAME_INT_CHECK_BONUS_TIME;
     gd_cave_clear_sounds(game->cave);
 
     // play cave finished sound
-    gd_sound_play(game->cave, GD_S_FINISHED, O_NONE, -1, -1);
+    if (!game->cave->no_time)
+      gd_sound_play(game->cave, GD_S_FINISHED, O_NONE, -1, -1);
     gd_sound_play_cave(game->cave);
   }
 
@@ -471,21 +516,21 @@ static GdGameState gd_game_main_int(GdGame *game, boolean allow_iterate, boolean
 
       if (game->player_move == GD_MV_STILL)
       {
-	game->player_move_stick = FALSE;
+	game->player_move_sticky = game_bd.sticky_movement_input;
       }
       else
       {
-	game->player_move_stick = TRUE;
+	game->player_move_sticky = FALSE;
 	game->player_move = GD_MV_STILL;
       }
 
       if (game->player_suicide == FALSE)
       {
-	game->player_suicide_stick = FALSE;
+	game->player_suicide_sticky = game_bd.sticky_movement_input;
       }
       else
       {
-	game->player_suicide_stick = TRUE;
+	game->player_suicide_sticky = FALSE;
 	game->player_suicide = FALSE;
       }
 
@@ -537,7 +582,11 @@ static GdGameState gd_game_main_int(GdGame *game, boolean allow_iterate, boolean
 	counter_next = GAME_INT_WAIT_BEFORE_COVER;
       }
 
-      if (game->cave->time / game->cave->timing_factor > 8)
+      // play "finished" sound loop when counting bonus time, but do not play it
+      // - for the last 9 seconds (while playing timeout sounds) unless using non-native sound
+      // - when not using cave time at all
+      if ((game->cave->time / game->cave->timing_factor > 8 || !game_has_native_sound_engine()) &&
+          !game->cave->no_time)
 	gd_sound_play(game->cave, GD_S_FINISHED, O_NONE, -1, -1); // play cave finished sound
 
       // play bonus sound
@@ -667,7 +716,7 @@ void play_game_func(GdGame *game, int action)
   boolean fire       = ((action & JOY_BUTTON)  != 0);
   boolean suicide    = ((action & KEY_SUICIDE) != 0);
 
-  if (game->player_move_stick || move_up || move_down || move_left || move_right) // no "fire"!
+  if (!game->player_move_sticky || move_up || move_down || move_left || move_right) // no "fire"!
   {
     // get movement
     game->player_move = gd_direction_from_keypress(move_up, move_down, move_left, move_right);
@@ -681,7 +730,7 @@ void play_game_func(GdGame *game, int action)
   if (game->player_move == GD_MV_STILL)
     game->player_fire = fire;
 
-  if (game->player_suicide_stick || suicide)
+  if (!game->player_suicide_sticky || suicide)
     game->player_suicide = suicide;
 
   // tell the interrupt "20ms has passed"
