@@ -18,7 +18,7 @@
 
 
 // !!! (can be removed later) !!!
-#define DO_GFX_SANITY_CHECK	TRUE
+#define DO_GFX_SANITY_CHECK	FALSE
 
 // distance to screen edge in cells when scrolling the screen
 #define SCROLL_EDGE_DISTANCE	4
@@ -91,7 +91,7 @@ void gd_init_play_area(void)
   slightly larger than the screen, in that case no scrolling is desirable
 */
 static boolean cave_scroll(int logical_size, int physical_size, int center, boolean exact,
-			   int *current, int *desired, int speed)
+			   int *current, int *desired, int speed, boolean infinite_scrolling)
 {
   int max = MAX(0, logical_size - physical_size);
   int edge_distance = SCROLL_EDGE_DISTANCE;
@@ -143,7 +143,22 @@ static boolean cave_scroll(int logical_size, int physical_size, int center, bool
     }
   }
 
-  *desired = MIN(MAX(0, *desired), max);
+  if (!infinite_scrolling)
+  {
+    // when not using infinite scrolling, always force visible area to be fully inside playfield
+    *desired = MIN(MAX(0, *desired), max);
+  }
+  else if (*desired < 0 || *desired > max)
+  {
+    int current_wrapped = *current - SIGN(*current) * logical_size;
+
+    // when using infinite scrolling, allow scrolling over playfield borders, but wrap if needed
+    if (ABS(*current - *desired) > ABS(current_wrapped - *desired))
+    {
+      *current = current_wrapped;
+      *desired = *current;
+    }
+  }
 
   if (*current < *desired)
   {
@@ -200,6 +215,11 @@ static boolean player_out_of_window(GdGame *game, int player_x, int player_y)
   if ((player_x * cell_size) < scroll_x ||
       (player_x * cell_size + cell_size - 1) > scroll_x + play_area_w)
   {
+    // when using infinite scrolling, ignore if player has just crossed playfield boundaries
+    if (game->cave->infinite_scrolling && (game->cave->player_x == game->cave->x1 ||
+                                           game->cave->player_x == game->cave->x2))
+      return FALSE;
+
     // but only do the wait, if the player SHOULD BE visible, ie. he is inside
     // the defined visible area of the cave
     if (game->cave->player_x >= game->cave->x1 &&
@@ -210,6 +230,11 @@ static boolean player_out_of_window(GdGame *game, int player_x, int player_y)
   if ((player_y * cell_size) < scroll_y ||
       (player_y * cell_size + cell_size - 1) > scroll_y + play_area_h)
   {
+    // when using infinite scrolling, ignore if player has just crossed playfield boundaries
+    if (game->cave->infinite_scrolling && (game->cave->player_y == game->cave->y1 ||
+                                           game->cave->player_y == game->cave->y2))
+      return FALSE;
+
     // but only do the wait, if the player SHOULD BE visible, ie. he is inside
     // the defined visible area of the cave
     if (game->cave->player_y >= game->cave->y1 &&
@@ -233,6 +258,9 @@ boolean gd_scroll(GdGame *game, boolean exact_scroll, boolean immediate)
   static int scroll_desired_x = 0, scroll_desired_y = 0;
   static int scroll_speed_last = -1;
   int player_x, player_y, visible_x, visible_y;
+  boolean infinite_scrolling = game->cave->infinite_scrolling;
+  boolean infinite_scrolling_x = (infinite_scrolling && game->cave->open_borders_horizontal);
+  boolean infinite_scrolling_y = (infinite_scrolling && game->cave->open_borders_vertical);
   boolean changed;
 
   // max scrolling speed depends on the speed of the cave.
@@ -258,6 +286,10 @@ boolean gd_scroll(GdGame *game, boolean exact_scroll, boolean immediate)
   if (player_out_of_window(game, player_x, player_y))
     scroll_speed *= 4;
 
+  // Kriss engine: scroll faster to start position when starting new game
+  if (game->cave->player_state == GD_PL_NOT_YET && game_bd.game->use_krissz_engine)
+    scroll_speed *= 4;
+
   // if scrolling started with player outside visible playfield area, keep faster scrolling
   if (scroll_speed_last > scroll_speed)
     scroll_speed = scroll_speed_last;
@@ -273,11 +305,11 @@ boolean gd_scroll(GdGame *game, boolean exact_scroll, boolean immediate)
   changed = FALSE;
 
   if (cave_scroll(visible_x, play_area_w, player_x * cell_size + cell_size / 2 - play_area_w / 2,
-		  exact_scroll, &scroll_x, &scroll_desired_x, scroll_speed))
+		  exact_scroll, &scroll_x, &scroll_desired_x, scroll_speed, infinite_scrolling_x))
     changed = TRUE;
 
   if (cave_scroll(visible_y, play_area_h, player_y * cell_size + cell_size / 2 - play_area_h / 2,
-		  exact_scroll, &scroll_y, &scroll_desired_y, scroll_speed))
+		  exact_scroll, &scroll_y, &scroll_desired_y, scroll_speed, infinite_scrolling_y))
     changed = TRUE;
 
   // if scrolling, we should update entire screen.
@@ -644,13 +676,52 @@ static int get_dirt_element(int element, int dir, boolean crumbled)
   return element;
 }
 
+static int get_screen_pos(int tile_pos, int playfield_tiles, int screen_size, int scroll_pos)
+{
+  int playfield_size = playfield_tiles * cell_size;
+  int screen_pos = tile_pos * cell_size - scroll_pos;
+
+  if (screen_pos <= -cell_size)
+    screen_pos += playfield_size;
+  else if (screen_pos >= screen_size)
+    screen_pos -= playfield_size;
+
+  return screen_pos;
+}
+
+static int get_screen_x(GdGame *game, int x)
+{
+  return get_screen_pos(x, game->cave->x2 - game->cave->x1 + 1, SXSIZE, scroll_x);
+}
+
+static int get_screen_y(GdGame *game, int y)
+{
+  return get_screen_pos(y, game->cave->y2 - game->cave->y1 + 1, SYSIZE, scroll_y);
+}
+
+static int getx(const GdCave *cave, const int x)
+{
+  if (!cave->open_borders_horizontal)
+    return MIN(MAX(0, x), cave->w - 1);
+
+  return (x + cave->w) % cave->w;
+}
+
+static int gety(const GdCave *cave, const int y)
+{
+  if (!cave->open_borders_vertical)
+    return MIN(MAX(0, y), cave->h - 1);
+
+  return (y + cave->h) % cave->h;
+}
+
 static void gd_drawcave_crumbled(Bitmap *dest, GdGame *game, int x, int y, boolean draw_masked)
 {
   void (*blit_bitmap)(Bitmap *, Bitmap *, int, int, int, int, int, int) =
     (draw_masked ? BlitBitmapMasked : BlitBitmap);
   GdCave *cave = game->cave;
-  int sx = x * cell_size - scroll_x;
-  int sy = y * cell_size - scroll_y;
+  int sx = get_screen_x(game, x);
+  int sy = get_screen_y(game, y);
   int border_size = cell_size / 8;
   int draw = game->drawing_buffer[y][x];
   int draw_last = game->last_drawing_buffer[y][x];
@@ -680,8 +751,8 @@ static void gd_drawcave_crumbled(Bitmap *dest, GdGame *game, int x, int y, boole
     int dir = dirs[i];
     int dx = gd_dx[dir];
     int dy = gd_dy[dir];
-    int xx = (x + dx + cave->w) % cave->w;
-    int yy = (y + dy + cave->h) % cave->h;
+    int xx = getx(cave, x + dx);
+    int yy = gety(cave, y + dy);
     int xoffset = (dx > 0 ? cell_size - border_size : 0);
     int yoffset = (dy > 0 ? cell_size - border_size : 0);
     int xsize = (dx == 0 ? cell_size : border_size);
@@ -722,8 +793,8 @@ static void gd_drawcave_tile(Bitmap *dest, GdGame *game, int x, int y, boolean d
   void (*blit_bitmap)(Bitmap *, Bitmap *, int, int, int, int, int, int) =
     (draw_masked ? BlitBitmapMasked : BlitBitmap);
   GdCave *cave = game->cave;
-  int sx = x * cell_size - scroll_x;
-  int sy = y * cell_size - scroll_y;
+  int sx = get_screen_x(game, x);
+  int sy = get_screen_y(game, y);
   int dir_from = game->dir_buffer_from[y][x];
   int dir_to = game->dir_buffer_to[y][x];
   int tile = game->element_buffer[y][x];
