@@ -10,6 +10,6 @@
 extern int AnimationPosTable[SP_MAX_PLAYFIELD_SIZE];
 extern byte AnimationSubTable[SP_MAX_PLAYFIELD_SIZE];
 
-void subDoGameStuff(void);
+void subDoGameStuff(byte);
 
 #endif // DOGAMESTUFF_H
