@@ -49,6 +49,7 @@ LevelDirTree	       *leveldir_current = NULL;
 int			level_nr;
 
 struct LevelSetInfo	levelset;
+struct LevelObjectInfo	levelobj;
 struct LevelStats	level_stats[MAX_LEVELS];
 
 DrawWindow	       *window = NULL;
@@ -70,6 +71,12 @@ int			FrameCounter = 0;
 // init/close functions
 // ============================================================================
 
+void InitProgramLogs(void)
+{
+  // always start with reliable default values
+  program.log_file = program.log_file_default = stdout;
+}
+
 void InitProgramInfo(char *command_filename,
 		     char *config_filename, char *userdata_subdir,
 		     char *program_basename, char *program_title,
@@ -101,7 +108,6 @@ void InitProgramInfo(char *command_filename,
   program.version_string = program_version_string;
 
   program.log_filename = getLogFilename(getLogBasename(program_basename));
-  program.log_file = program.log_file_default = stdout;
 
   program.api_thread_count = 0;
 
@@ -718,6 +724,9 @@ static boolean CheckDrawingArea(int x, int y, int draw_mask)
   if ((draw_mask & REDRAW_DOOR_3) && IN_GFX_DOOR_3(x, y))
     return TRUE;
 
+  if (draw_mask & REDRAW_REQUEST)
+    return TRUE;
+
   return FALSE;
 }
 
