@@ -37,12 +37,13 @@ void DisableDrawingText(void)
   text_drawing_enabled = FALSE;
 }
 
-void InitFontInfo(struct FontBitmapInfo *font_bitmap_info, int num_fonts,
+void InitFontInfo(struct FontBitmapInfo *font_bitmap_info, int num_fonts, boolean uppercase_only,
 		  int (*select_font_function)(int),
 		  int (*get_font_from_token_function)(char *),
 		  char * (*get_token_from_font_function)(int))
 {
   gfx.num_fonts = num_fonts;
+  gfx.uppercase_only = uppercase_only;
   gfx.font_bitmap_info = font_bitmap_info;
   gfx.select_font_function = select_font_function;
   gfx.get_font_from_token_function = get_font_from_token_function;
@@ -103,11 +104,17 @@ static int getFontCharPosition(int font_nr, char c)
   struct FontBitmapInfo *font = &gfx.font_bitmap_info[font_bitmap_id];
   int font_pos = (unsigned char)c - 32;
 
+  // map to uppercase characters if requested
+  if (font->uppercase_only || gfx.uppercase_only)
+    c = MAP_FONT_UPPERCASE(c);
+
   // map some special characters to their ascii values in default font
   if (font->num_chars == DEFAULT_NUM_CHARS_PER_FONT)
     font_pos = MAP_FONT_ASCII(c) - 32;
   else if (font->num_chars == NUM_CHARS_PER_FONT_EXT)
     font_pos = MAP_FONT_ASCII_EXT(c) - 32;
+  else if (font->num_chars == NUM_CHARS_PER_FONT_BDX)
+    font_pos = MAP_FONT_ASCII_BDX(c) - 32;
 
   // this allows dynamic special characters together with special font
   if (font_pos < 0 || font_pos >= font->num_chars)
@@ -416,6 +423,15 @@ char *GetTextBufferFromFile(char *filename, int max_lines)
     buffer = body_latin1;
   }
 
+  if (hasTabs(buffer))
+  {
+    char *buffer_expanded_tabs = getExpandedTabs(buffer);
+
+    checked_free(buffer);
+
+    buffer = buffer_expanded_tabs;
+  }
+
   return buffer;
 }
 
